(in-package "BLUDISTE")

(provide "aktivni_objekt")

(require "konstanty" "konstanty")

(require "pozice" "pozice")

(require "smer" "smer")



; ***** aktivni objekt *****

; ========================



;spolecny predek pro vsechny aktivni objekty: 

;   postavy, prisery, strely, monitory



(defclass trida_aktivni_objekt () 

  (

    (pozice :type trida_pozice :accessor slot_pozice 

            :initform (make-instance 'trida_pozice :x 0 :y 0) 

            :initarg :pozice)

    (smer_pohledu :type symbol :accessor slot_smer_pohledu 

                  :initform 'nahoru :initarg :smer_pohledu)

    (pocet_zivotu :type integer :accessor slot_pocet_zivotu :initform 1

                  :initarg :pocet_zivotu)

    (tahovy_objekt :type trida_tahovy_objekt :accessor slot_tahovy_objekt 

                   :initform nil :initarg :tahovy_objekt)

    (log :type list :accessor slot_log :initform nil :initarg :log)

  )

)



(defmethod stav_z_logu((aktivni_objekt trida_aktivni_objekt)

                       (cislo_tahu integer))

  (let ((log (slot_log aktivni_objekt)))

    (cond ((null log) nil)

          (t (let* ((zaznam (car log))

                    (pozice_xy (car zaznam))

                    (pozice_x (car pozice_xy))

                    (pozice_y (cadr pozice_xy))

                    (pozice (make-instance 'trida_pozice :x pozice_x :y pozice_y))

                    (smer_pohledu (cadr zaznam))

                    (pocet_zivotu (caddr zaznam))

                   )

               (setf (slot_pozice aktivni_objekt) pozice)

               (setf (slot_smer_pohledu aktivni_objekt) smer_pohledu)

               (setf (slot_pocet_zivotu aktivni_objekt) pocet_zivotu)

               (setf (slot_log aktivni_objekt) (cdr log))

             )

          )

    )

  )

)



(defmethod pridej_log((aktivni_objekt trida_aktivni_objekt))

  (let* ((pozice (slot_pozice aktivni_objekt))

         (pozice2 (pozice_xy pozice))

         (smer_pohledu (slot_smer_pohledu aktivni_objekt))

         (pocet_zivotu (slot_pocet_zivotu aktivni_objekt))

         (zaznam (list pozice2 smer_pohledu pocet_zivotu))

         (log (slot_log aktivni_objekt))

         (novy_log (cons zaznam log)))

    (setf (slot_log aktivni_objekt) novy_log)

  )

)



(defmethod odecist_zivot ((postava trida_aktivni_objekt))

  (setf (slot_pocet_zivotu postava) 

        (1- (slot_pocet_zivotu postava)))

)



(defmethod pridat_zivot ((postava trida_aktivni_objekt))

  (setf (slot_pocet_zivotu postava) 

        (1+ (slot_pocet_zivotu postava)))

)



(defmethod start ((aktivni_objekt trida_aktivni_objekt)) 

  (declare (ignore aktivni_objekt))

  nil

)



(defmethod krok ((aktivni_objekt trida_aktivni_objekt)) 

  (declare (ignore aktivni_objekt))

  nil

)



(defmethod konec ((aktivni_objekt trida_aktivni_objekt)) 

  (declare (ignore aktivni_objekt))

  nil

)





(defmethod nova_pozice((postava trida_aktivni_objekt) (smer symbol))

  (let* ((pozice (slot_pozice postava))

         (nova_pozice_relativne (nova_pozice_relativne smer))

         (nova_pozice_relativne_transformovana

           (transformace_pohled nova_pozice_relativne

                                (slot_smer_pohledu postava)))

        )

    (soucet pozice 

            nova_pozice_relativne_transformovana)

  )

)



(defmethod otoceny_pohled((postava trida_aktivni_objekt)

                          (prisera trida_aktivni_objekt))

  (let* ((postava_pohled (slot_smer_pohledu postava))

         (prisera_pohled (slot_smer_pohledu prisera))

         (vektor_pohledu_prisera (transformace_pohled 

                                    (make-instance 'trida_pozice 

                                                   :x 0 :y 1)

                                    prisera_pohled))

         (otoceny_vektor (transformace_pohled vektor_pohledu_prisera

                                              postava_pohled))

        )

    (smer_pohledu_z_vektoru otoceny_vektor)

  )

)



(defmethod vytvor_cisty_rozdil((postava trida_aktivni_objekt)

                               (prisera trida_aktivni_objekt))

  (rozdil (slot_pozice prisera) 

          (slot_pozice postava))

)



(defmethod vytvor_relativni_pozici((postava trida_aktivni_objekt)

                                   (prisera trida_aktivni_objekt))

  (transformace_pohled (vytvor_cisty_rozdil postava prisera)

                       (slot_smer_pohledu postava))

)



(defmethod test_rovnost((postava trida_aktivni_objekt)

                        (prisera trida_aktivni_objekt))

  (rovnost (slot_pozice prisera) 

           (slot_pozice postava))

)



(defmethod test_rozdil((postava trida_aktivni_objekt)

                       (prisera trida_aktivni_objekt))

  (let ((rozdil (vytvor_cisty_rozdil postava prisera)))

    (and (<= (abs (slot_x rozdil)) max_viditelnost)

         (<= (abs (slot_y rozdil)) max_viditelnost))

  )

)



(defmethod test_znamenka((postava trida_aktivni_objekt)

                         (prisera trida_aktivni_objekt))

  (let* ((rozdil (vytvor_cisty_rozdil postava prisera))

         (rozdil2 (transformace_pohled rozdil 

                                       (slot_smer_pohledu postava)))

       )

    (cond ((>= (slot_y rozdil2) 0) t)

          (t nil)

    )

  )

)





; ***** prace s aktivnimi objekty *****



;test zda je nejaka z postav na pozici

(defmethod test_aktivni_objekty_pozice((pozice trida_pozice)

                                       (aktivni_objekty list))

  (some

    #'(lambda (aktivni_objekt) (rovnost pozice 

                                        (slot_pozice aktivni_objekt)))

    aktivni_objekty

  )

)



;vybrat postavy na radku

(defmethod aktivni_objekty_na_radku((cislo_radku integer)

                                    (aktivni_objekty list))

  (remove-if-not

    #'(lambda (aktivni_objekt)

      (eql cislo_radku (slot_y (slot_pozice aktivni_objekt)))

    )

    aktivni_objekty

  )

)





;vybrat postavy na dane pozici

(defmethod aktivni_objekty_na_pozici((pozice trida_pozice)

                                     (aktivni_objekty list))

  (remove-if-not

    #'(lambda (aktivni_objekt) (rovnost pozice 

                                        (slot_pozice aktivni_objekt)))

    aktivni_objekty

  )

)

