(defpackage "BLUDISTE")



(in-package "BLUDISTE")

(provide "hra")

(require "tahovy_objekt" "tahovy_objekt")

(require "prisera" "prisera")

(require "strela" "strela")

(require "monitor" "monitor")

(require "viditelna_mapa" "viditelna_mapa")

(require "viditelne_objekty" "viditelne_objekty")



; --------------------------------------------------------------

; ***** hra *****

; ===============



;definice objektu predstavujiciho stav hry



(defclass trida_hra()

  (

    (mapa :type trida_mapa :accessor slot_mapa :initarg :mapa)

    (postavy :type list :accessor slot_postavy :initform nil

             :initarg :postavy)

    (prisery :type list :accessor slot_prisery :initform nil

             :initarg :prisery)

    (strely :type list :accessor slot_strely :initform nil

            :initarg :strely)

    (konec :type string :accessor slot_konec :initform nil)

    (cislo_tahu :type integer :accessor slot_cislo_tahu :initform 0

                :initarg :cislo_tahu)

    (maximum_tahu :type integer :accessor slot_maximum_tahu :initform 10000

                  :initarg :maximum_tahu)

    (monitory :type list :accessor slot_monitory :initform nil

              :initarg :monitory)

    (log_vyrazene_prisery :type list :accessor slot_log_vyrazene_prisery

                          :initform nil)

    (log_vyrazene_strely :type list :accessor slot_log_vyrazene_strely

                          :initform nil)

  )

)



(require "provadeni_tahu" "provadeni_tahu")

(require "kontrola" "kontrola")





(defmethod nastav_konec_hry((hra trida_hra) (zprava string))

  (cond

    ((slot_konec hra) nil)

    (t (setf (slot_konec hra) zprava))

) )



;pricti jednicku k poctu tahu a vrat true, pokud nebylo prekroceno maximum

(defmethod hra_zvetsit_cislo_tahu((hra trida_hra))

  (setf (slot_cislo_tahu hra) (1+ (slot_cislo_tahu hra)))

  (<= (slot_cislo_tahu hra) (slot_maximum_tahu hra))

)



;odstranit objekt ze hry

(defmethod odstranit_priseru((hra trida_hra) (prisera trida_prisera))

  (setf (slot_prisery hra) (delete prisera (slot_prisery hra)))

  (setf (slot_log_vyrazene_prisery hra)

        (cons (reverse (slot_log prisera))

              (slot_log_vyrazene_prisery hra)

              ))

)



(defmethod test_odstranit_priseru((hra trida_hra) (prisera trida_prisera))

  (cond

    ((zerop (slot_pocet_zivotu prisera))

      (odstranit_priseru hra prisera)

  ) )

)



(defmethod odstranit_strelu((hra trida_hra) (strela trida_strela))

  (setf (slot_strely hra) (delete strela (slot_strely hra)))

  (setf (slot_log_vyrazene_strely hra)

        (cons (reverse (slot_log strela))

              (slot_log_vyrazene_strely hra)

              ))

)



(defmethod test_odstranit_strelu((hra trida_hra) (strela trida_strela))

  (cond

    ((< (slot_pocet_zivotu strela) 1)

      (odstranit_strelu hra strela)

  ) )

)



;pridat strelu do hry

(defmethod pridat_strelu((hra trida_hra) (strela trida_strela))

  (setf (slot_strely hra) (nconc (slot_strely hra) (list strela)))

)



;pridat monitor do hry

(defmethod pridat_monitor((hra trida_hra) (monitor trida_monitor))

  (setf (slot_monitory hra) (nconc (slot_monitory hra) (list monitor)))

)





; zavolani dane funkce pro vsechny objekty ve hre



(defmethod pro_vsechny((hra trida_hra) (funkce function))

  (mapc funkce (slot_postavy hra))

  (mapc funkce (slot_prisery hra))

  (mapc funkce (slot_strely hra))

  (mapc funkce (slot_monitory hra))

)



(defmethod pro_strely((hra trida_hra) (funkce function))

  (mapc funkce (slot_strely hra))

)



(defmethod pro_prisery((hra trida_hra) (funkce function))

  (mapc funkce (slot_prisery hra))

)



(defmethod pro_postavy((hra trida_hra) (funkce function))

  (mapc funkce (slot_postavy hra))

)



(defmethod pro_monitory ((hra trida_hra) (funkce function))

  (mapc funkce (slot_monitory hra))

)



(defmethod hra_init((hra trida_hra))

  (pro_monitory hra

    #'(lambda (monitor) (setf (slot_hra monitor) hra))

  )

  (setf (slot_konec hra) nil)

)





; spusteni hry a provadeni tahu

; vykonani jednoho tahu (hra_jeden_tah) je v souboru tah.lsp



(defmethod spustit((hra trida_hra))

  (hra_init hra)

  (pro_vsechny hra #'start)

  (pro_vsechny hra #'krok)

  (proved_jeden_tah hra)

  (pro_vsechny hra #'konec)

  nil

)



;provedeni jednoho tahu

(defmethod proved_jeden_tah ((hra trida_hra))

  (cond ((not (hra_zvetsit_cislo_tahu hra))

    (nastav_konec_hry hra "PREKROCEN MAXIMALNI POCET TAHU")

  ))

  (cond ((slot_konec hra) nil)

        (t

         (tah_vsechny hra (slot_strely hra) #'kontrola_tahu_strela)

         (tah_vsechny hra (slot_prisery hra) #'kontrola_tahu_prisera)

         (pro_vsechny hra #'krok)

         (tah_vsechny hra (slot_postavy hra) #'kontrola_tahu_postava)

         (pro_strely hra #'odecist_zivot)

         (pro_strely hra #'(lambda(strela) 

                             (test_odstranit_strelu hra strela)))

         (proved_jeden_tah hra)

        )

  )

)



;provedeni tahu vsemi objekty ze seznamu

(defmethod tah_vsechny((hra trida_hra) (aktivni_objekty list) 

                       (kontrola function))

  (mapc #'(lambda (aktivni_objekt)

          (let ((co_delat (ziskej_tah hra aktivni_objekt)))

            (cond ((funcall kontrola hra aktivni_objekt co_delat)

                     (proved_tah hra aktivni_objekt co_delat)

                     (pridej_log aktivni_objekt))

        ) ) )

        aktivni_objekty

  ) 

)



;ziskani dalsiho tahu od postavy, strely nebo prisery

(defmethod ziskej_tah((hra trida_hra) (aktivni_objekt trida_aktivni_objekt))

  (let* ((vidi (or (typep aktivni_objekt 'trida_postava)

                   (typep aktivni_objekt 'trida_strela)))

         (viditelna_mapa (cond (vidi

                                  (viditelna_mapa (slot_mapa hra) aktivni_objekt))

                               (t (make-instance 'trida_mapa :pole #(#())))

                         ))

        )

    (tahova_funkce

      (slot_tahovy_objekt aktivni_objekt)

      (make-instance 'trida_parametry_tahu

        :mapa viditelna_mapa

        :postavy (viditelne_objekty aktivni_objekt 

                                    (slot_postavy hra) viditelna_mapa)

        :prisery (viditelne_objekty aktivni_objekt 

                                    (slot_prisery hra) viditelna_mapa)

        :strely (viditelne_objekty aktivni_objekt 

                                   (slot_strely hra) viditelna_mapa)

        :cislo_tahu (slot_cislo_tahu hra)

        :maximum_tahu (slot_maximum_tahu hra)

        :vlastni_postava (viditelne_parametry_postavy aktivni_objekt)

      )

    )

  )

)



(defmethod log_aktivni_objekty((aktivni_objekty list))

  (mapcar (lambda(postava) 

            (reverse (slot_log postava))

          )

          aktivni_objekty

  )

)



(defmethod zapis_log((hra trida_hra))

  (let* ((log_postavy (log_aktivni_objekty (slot_postavy hra)))

         (log_prisery (log_aktivni_objekty (slot_prisery hra)))

         (log_strely (log_aktivni_objekty (slot_strely hra)))

         (log_vyrazene_prisery (slot_log_vyrazene_prisery hra))

         (log_vyrazene_strely (slot_log_vyrazene_strely hra))

         (log_prisery2 (nconc log_prisery log_vyrazene_prisery))

         (log_strely2 (nconc log_strely log_vyrazene_strely))

         (jmeno_souboru "log.lsp")

         (s (open jmeno_souboru :direction :output :if-exists :supersede))

       )

    (zapis_log_hlavicka s)



    (format s "    :mapa (make-instance 'trida_mapa :pole~%")

    (zapis_mapu (slot_mapa hra) s)

    (format s ")~%")



    (format s "    :log_postavy~%")

    (zapis_log_seznam s "                 " log_postavy)



    (format s "    :log_prisery~%")

    (zapis_log_seznam s "                 " log_prisery2)



    (format s "    :log_strely~%")

    (zapis_log_seznam s "                " log_strely2)



    (zapis_log_konec s)

    (close s)

  )

)



(defun zapis_log_hlavicka(vystup)

  (format vystup "(in-package \"BLUDISTE\")~%")

  (format vystup "~%")

  (format vystup "(require \"log_viewer\" \"log_viewer\")~%")

  (format vystup "~%")

  (format vystup "(setq priklad_trida_monitor 'trida_monitor_graficky)~%")

  (format vystup ";(setq priklad_trida_monitor 'trida_monitor_textove)~%")

  (format vystup "~%")

  (format vystup "(spustit~%")

  (format vystup "  (make-instance 'trida_log_viewer~%")

)



(defun zapis_log_seznam(vystup odsazeni seznam)

  (format vystup "~A'(~%" odsazeni)

  (mapc #'(lambda(prvek)

            (format vystup "~A  ~A~%" odsazeni prvek)

          )

        seznam

  )

  (format vystup "~A )~%" odsazeni)

)



(defun zapis_log_konec(vystup)

  (format vystup "    :monitory (list (make-instance priklad_trida_monitor))~%")

  (format vystup "  )~%")

  (format vystup ")~%")

)

