(in-package "BLUDISTE")

(provide "kontrola")

(require "hra" "hra")

(require "postava" "postava")

(require "prisera" "prisera")

(require "strela" "strela")

(require "pozice" "pozice")

(require "tah" "tah")

(require "aktivni_objekt" "aktivni_objekt")



; --------------------------------------------------------------

; kontrola pohybu a kolizi

; ========================





; ***** reseni udalosti (kolizi) *****



;uspesny konec hry - postava dosla do vychodu

(defmethod vychod ((hra trida_hra)

                   (postava trida_postava)

                   (nova_pozice trida_pozice))

  (declare (ignore postava nova_pozice))

  (nastav_konec_hry hra "VYCHOD")

  t

)



;odecteni zivota postavam, ktere se dostaly do kolize

(defmethod kolize_s_postavou((hra trida_hra) (postavy list))

  (mapc

    #'(lambda (postava)

      (odecist_zivot postava)

      (cond ((zerop (slot_pocet_zivotu postava))

        (nastav_konec_hry hra "KOLIZE - posledni zivot!"))

    ) )

    postavy

  )

  nil

)



;doslo ke kolizi postavy s postavou/priserou/strelou/zdi

(defmethod kolize_postava((hra trida_hra) 

                          (postava trida_postava)

                          (nova_pozice trida_pozice))

  (declare (ignore nova_pozice))

  (kolize_s_postavou hra (list postava))



  (let ((prisery (aktivni_objekty_na_pozici nova_pozice

                                            (slot_prisery hra))))

    (mapc #'(lambda (prisera)

             (odecist_zivot prisera)

             (test_odstranit_priseru hra prisera)

          )

          prisery)

  )

)



;doslo ke kolizi prisery s postavou

(defmethod kolize_prisera_postava((hra trida_hra) 

                                  (prisera trida_prisera)

                                  (nova_pozice trida_pozice))

  (declare (ignore prisera))

  (kolize_s_postavou hra

    (aktivni_objekty_na_pozici nova_pozice (slot_postavy hra))

  )

  (odecist_zivot prisera)

  (test_odstranit_priseru hra prisera)

)



;doslo ke kolizi strely s postavou

(defmethod kolize_strela_postava((hra trida_hra) 

                                 (strela trida_strela)

                                 (nova_pozice trida_pozice))

  (declare (ignore strela))

  (kolize_s_postavou hra

    (aktivni_objekty_na_pozici nova_pozice (slot_postavy hra))

  )

)



;doslo ke kolizi prisery se strelou

(defmethod kolize_prisera_strela((hra trida_hra)

                                 (prisera trida_prisera)

                                 (nova_pozice trida_pozice))

  (let ((strely (aktivni_objekty_na_pozici nova_pozice

                                           (slot_strely hra))))

    (odecist_zivot prisera)

    (test_odstranit_priseru hra prisera)

    (mapc #'(lambda (strela)

            (postava_pridat_bod (slot_vlastnik strela))

            (odstranit_strelu hra strela))

          strely)

    nil

  )

)



;doslo ke kolizi strely s priserou

(defmethod kolize_strela_prisera((hra trida_hra)

                                 (strela trida_strela)

                                 (nova_pozice trida_pozice))

  (let ((prisery (aktivni_objekty_na_pozici nova_pozice

                                            (slot_prisery hra))))

    (mapc #'(lambda (prisera)

             (odecist_zivot prisera)

             (postava_pridat_bod (slot_vlastnik strela))

             (test_odstranit_priseru hra prisera)

          )

          prisery)

    (odstranit_strelu hra strela)

    t

  )

)



;doslo ke kolizi strely se zdi

(defmethod kolize_strela_zed((hra trida_hra)

                             (strela trida_strela)

                             (nova_pozice trida_pozice))

  (declare (ignore nova_pozice))

  (odstranit_strelu hra strela)

  nil

)



;doslo ke kolizi dvou strel

(defmethod kolize_strela_strela((hra trida_hra)

                                (strela trida_strela)

                                (nova_pozice trida_pozice))

  (let ((strely (aktivni_objekty_na_pozici nova_pozice

                                           (slot_strely hra))))

    (mapc #'(lambda(strela1) (odstranit_strelu hra strela1)) strely)

    (odstranit_strelu hra strela)

    nil

  )

)



(defmethod nepovolit((hra trida_hra)

                     (postava trida_aktivni_objekt)

                     (nova_pozice trida_pozice))

  (declare (ignore hra postava nova_pozice))

  nil

)



;kontrola tahu

(defmethod kontrola_tahu((hra trida_hra)

                         (postava trida_aktivni_objekt)

                         (tah trida_tah)

                         (akce_zed function)

                         (akce_vychod function)

                         (akce_postava function)

                         (akce_prisera function)

                         (akce_strela function))

  (let* ((akce (slot_akce tah))

         (nova_pozice (nova_pozice postava akce))

         (stav_pole (get_pole (slot_mapa hra) nova_pozice))

         (stav_postava (test_aktivni_objekty_pozice nova_pozice

                                                    (slot_postavy hra)))

         (stav_prisera (test_aktivni_objekty_pozice nova_pozice

                                                    (slot_prisery hra)))

         (stav_strela (test_aktivni_objekty_pozice nova_pozice

                                                   (slot_strely hra)))

        )

    (cond

          ((member akce '(otoc_doprava otoc_doleva otoc_dozadu)) t)

          ((eql akce 'konec)

            (nastav_konec_hry hra "Ukonceno na zadost postavy")

            nil

          )

          ((eql akce 'strela) t)

          ((eql akce 'stop) t)

          ((rovnost (slot_pozice postava) 

                    nova_pozice) t)

          ((eql stav_pole 'zed) (funcall akce_zed hra postava nova_pozice))

          ((eql stav_pole 'vychod) (funcall akce_vychod hra postava nova_pozice))

          (stav_postava (funcall akce_postava hra postava nova_pozice))

          (stav_prisera (funcall akce_prisera hra postava nova_pozice))

          (stav_strela (funcall akce_strela hra postava nova_pozice))

          (t t)

    )

) )



;kontrola tahu pro postavu

(defmethod kontrola_tahu_postava((hra trida_hra)

                                 (postava trida_postava)

                                 (tah trida_tah))

  (kontrola_tahu hra postava tah #'kolize_postava #'vychod

      #'kolize_postava #'kolize_postava #'kolize_postava)

)



;kontrola tahu pro priseru

(defmethod kontrola_tahu_prisera((hra trida_hra) 

                                 (prisera trida_prisera)

                                 (tah trida_tah))

  (kontrola_tahu hra prisera tah #'nepovolit #'nepovolit

      #'kolize_prisera_postava #'nepovolit #'kolize_prisera_strela)

)



;kontrola tahu pro strelu

(defmethod kontrola_tahu_strela((hra trida_hra)

                                (strela trida_strela)

                                (tah trida_tah))

  (kontrola_tahu hra strela tah #'kolize_strela_zed #'kolize_strela_zed

      #'kolize_strela_postava #'kolize_strela_prisera #'kolize_strela_strela)

)

