(in-package "BLUDISTE")

(provide "tisk_graficky")

(require "hra" "hra")

(require "graficke_rozhrani" "graficke_rozhrani")

(require "aktivni_objekt" "aktivni_objekt")

(require "pozice" "pozice")



; --------------------------------------------------------------

; graficke zobrazovaci funkce

; ===========================



(defmethod barva_policka((pozice trida_pozice)

                         (hra trida_hra)

                         (postavy_radek list)

                         (prisery_radek list)

                         (strely_radek list))

  (let ((policko (get_pole (slot_mapa hra) pozice)))

    (cond ((test_aktivni_objekty_pozice pozice postavy_radek)

             (list :green3 (car (aktivni_objekty_na_pozici pozice 

                                                          postavy_radek))))

          ((test_aktivni_objekty_pozice pozice prisery_radek)

             (list :red (car (aktivni_objekty_na_pozici pozice

                                                        prisery_radek))))

          ((test_aktivni_objekty_pozice pozice strely_radek)

             (list :blue (car (aktivni_objekty_na_pozici pozice 

                                                         strely_radek))))

          ((eql policko 'zed) (list :brown nil))

          ((eql policko 'chodba) (list :white nil))

          ((eql policko 'vychod) (list :magenta nil))

          ((eql policko 'strela) (list :cyan nil))

          ((eql policko 'klic) (list :yellow nil))

          ((null policko) (list :gray nil))

    )

  )

)



(defmethod tisk_postavy(pane 

                        (postava trida_aktivni_objekt)

                        (x integer)

                        (y integer)

                        (velikost integer)

                        font)

  (cond ((null postava) nil)

        (t

  (let* ((pulka (/ velikost 2))

;         (s (- (/ velikost 2) (/ velikost 8)))

         (s pulka)

         (s2 (/ velikost 6))

         (x_pul (+ x pulka))

         (y_pul (+ y pulka))

         (zivotu (slot_pocet_zivotu postava))

         (pohled (slot_smer_pohledu postava))

        )

  (cond ((eql 'dolu pohled)

           (gp:draw-line pane x_pul (- y_pul s)

                              x_pul (+ y_pul s))

           (gp:draw-line pane (- x_pul s) (- y_pul s2)

                              x_pul (+ y_pul s))

           (gp:draw-line pane (+ x_pul s) (- y_pul s2)

                              x_pul (+ y_pul s))

        )

        ((eql 'nahoru pohled)

           (gp:draw-line pane x_pul (+ y_pul s)

                              x_pul (- y_pul s))

           (gp:draw-line pane (- x_pul s) (+ y_pul s2)

                              x_pul (- y_pul s))

           (gp:draw-line pane (+ x_pul s) (+ y_pul s2)

                              x_pul (- y_pul s))

        )

        ((eql 'doprava pohled)

           (gp:draw-line pane (- x_pul s) y_pul

                              (+ x_pul s) y_pul)

           (gp:draw-line pane (- x_pul s2) (- y_pul s)

                              (+ x_pul s) y_pul)

           (gp:draw-line pane (- x_pul s2) (+ y_pul s)

                              (+ x_pul s) y_pul)

        )

        ((eql 'doleva pohled)

           (gp:draw-line pane (- x_pul s) y_pul

                              (+ x_pul s) y_pul)

           (gp:draw-line pane (+ x_pul s2) (- y_pul s)

                              (- x_pul s) y_pul)

           (gp:draw-line pane (+ x_pul s2) (+ y_pul s)

                              (- x_pul s) y_pul)

        )

  )

  (cond ((null zivotu) nil)

        (t (gp:draw-string pane

                           (format nil "~A" zivotu)

                           x (+ y velikost) :foreground :yellow

                           :font font

           )

        )

  )

  )

)))



(defmethod tisk_policka_graficky((pozice trida_pozice) 

                                 (hra trida_hra)

                                 pane

                                 (postavy_radek list)

                                 (prisery_radek list)

                                 (strely_radek list)

                                 (velikost integer)

                                 font)

  (let* ((barva_policka

          (barva_policka pozice hra postavy_radek prisery_radek strely_radek))

         (barva (car barva_policka))

         (postava_na_policku (cadr barva_policka))

         (x (* velikost (slot_x pozice)))

         (y (* velikost (slot_y pozice)))

        )

    (gp:draw-rectangle pane x y velikost velikost

                       :filled t :foreground barva)

    (gp:draw-rectangle pane x y velikost velikost

                       :filled nil :foreground :black)

    (cond ((not (null postava_na_policku))

           (tisk_postavy pane postava_na_policku x y velikost font))

          )

  )

)



(defmethod tisk_radku_graficky((cislo_radku integer)

                               (hra trida_hra)

                               pane

                               (velikost integer)

                               font)

  (let ((postavy_radek (aktivni_objekty_na_radku cislo_radku

                                                 (slot_postavy hra)))

        (prisery_radek (aktivni_objekty_na_radku cislo_radku

                                                 (slot_prisery hra)))

        (strely_radek (aktivni_objekty_na_radku cislo_radku

                                                (slot_strely hra)))

       )

    (mapc #'(lambda (cislo_sloupce)

              (tisk_policka_graficky (make-instance 'trida_pozice 

                                                    :x cislo_sloupce :y cislo_radku)

                                     hra pane postavy_radek prisery_radek

                                     strely_radek velikost font)

            )

          (cisla (sirka (slot_mapa hra)))

    )

  )

)



(defmethod tisk_mapy_graficky((hra trida_hra) 

                              pane

                              (velikost integer))

  (let ((font (gp:find-best-font pane

                  (gp:make-font-description :family "times"

                                            :size (/ velikost 4/3)

                                            :weight :medium

                                            :slant :roman)))

       )

    (mapc (lambda (cislo_radku)

            (tisk_radku_graficky cislo_radku hra pane velikost font)

          )

          (cisla (vyska (slot_mapa hra)))

    )

  )

  nil

)





(defun nakresli_viditelnou_mapu(pane x y width height)

  (declare (ignore x y width height))

  (let* ((velikost 15))

    (tisk_mapy_graficky (slot_hra (capi:element-interface pane)) pane velikost)

  )

)





(defun nakresli_mapu(pane x y width height)

  (declare (ignore x y width height))

  (let* ((velikost 10))

    (tisk_mapy_graficky (slot_hra (capi:element-interface pane)) pane velikost)

  )

)





(defmethod prekresli_mapu((interface interface_bludiste))

  (gp:invalidate-rectangle (slot-value interface 'mapa))

  (setf (capi:title-pane-text (slot-value interface 'cislo_tahu))

     (format nil "~D" (slot_cislo_tahu (slot_hra interface))))

  (setf (capi:title-pane-text (slot-value interface 'max_tahu))

     (format nil "~D" (slot_maximum_tahu (slot_hra interface))))

)





(defmethod prekresli_viditelnou_mapu ((interface interface_postavicka))

  (gp:invalidate-rectangle (slot-value interface 'mapa))

  (setf (capi:title-pane-text (slot-value interface 'cislo_tahu))

     (format nil "~D" (slot_cislo_tahu (slot_hra interface))))

  (setf (capi:title-pane-text (slot-value interface 'max_tahu))

     (format nil "~D" (slot_maximum_tahu (slot_hra interface))))

  (setf (capi:title-pane-text (slot-value interface 'pocet_zivotu))

     (format nil "~D" (slot_pocet_zivotu (slot_postava interface))))

  (setf (capi:title-pane-text (slot-value interface 'pocet_klicu))

     (format nil "~D" (slot_pocet_klicu (slot_postava interface))))

  (setf (capi:title-pane-text (slot-value interface 'pocet_strel))

     (format nil "~D" (slot_pocet_strel (slot_postava interface))))

  (setf (capi:title-pane-text (slot-value interface 'pocet_bodu))

     (format nil "~D" (slot_pocet_bodu (slot_postava interface))))

)



