(in-package "BLUDISTE")
(provide "tah_textove")
(require "tahovy_objekt" "tahovy_objekt")
(require "parametry_tahu" "parametry_tahu")
(require "tah" "tah")

(defclass trida_tahovy_objekt_postava_valeko(trida_tahovy_objekt)
  (
    (parametry    :type trida_parametry_tahu :accessor slot_parametry  
                  :initform nil)
    (pozice       :type trida_pozice         :accessor slot_pozice     
                  :initform (make-instance 'trida_pozice :x 0 :y 0))
    (pohled       :type symbol               :accessor slot_pohled
                  :initform 'nahoru)
    (posl_pozice  :type trida_pozice         :accessor slot_posl_pozice
                  :initform (make-instance 'trida_pozice :x 0 :y 0))
    (posl_akce    :type symbol               :accessor slot_posl_akce
                  :initform nil)
    (posl_otoceni :type symbol               :accessor slot_posl_otoceni
                  :initform nil)
    (navstiveno   :type simple-vector        :accessor slot_navstiveno
                  :initform (make-array '(200 200) :initial-element 0))
    (x_max        :type integer              :accessor slot_x_max
                  :initform 0)
    (x_min        :type integer              :accessor slot_x_min
                  :initform 0)
    (y_max        :type integer              :accessor slot_y_max
                  :initform 0)
    (y_min        :type integer              :accessor slot_y_min
                  :initform 0)
    (posl_zbyva   :type integer              :accessor slot_posl_zbyva
                  :initform 0)
    (tahu         :type integer              :accessor slot_tahu
                  :initform 0)
  )
)

; vyrobi instanci tridy trida_pozice ze smeru pohybu
(defun vektor(smer velikost)
  (cond ((eq smer 'dopredu) (make-instance 'trida_pozice :x  0 :y      velikost))
        ((eq smer 'dozadu)  (make-instance 'trida_pozice :x  0 :y (- 0 velikost)))
        ((eq smer 'doleva)  (make-instance 'trida_pozice :x (- 0 velikost) :y  0))
        ((eq smer 'doprava) (make-instance 'trida_pozice :x  velikost      :y  0))
        ((eq smer 'dopredu_doprava)
                            (make-instance 'trida_pozice :x  velikost      :y  velikost))
        ((eq smer 'dopredu_doleva)
                            (make-instance 'trida_pozice :x (- 0 velikost) :y  velikost))
        (nil)
  )
)

; otoci dany vektor podle daneho smeru pohledu
(defun otoc_vektor(pohled vektor)
  (let ((x (slot_x vektor))
        (y (slot_y vektor)))
       (cond ((eq pohled 'nahoru)  (make-instance 'trida_pozice :x (*  1 x) :y (*  1 y) ))
             ((eq pohled 'dolu)    (make-instance 'trida_pozice :x (* -1 x) :y (* -1 y) ))
             ((eq pohled 'doleva)  (make-instance 'trida_pozice :x (* -1 y) :y (*  1 x) ))
             ((eq pohled 'doprava) (make-instance 'trida_pozice :x (*  1 y) :y (* -1 x) ))
             (nil)
        )
  )
)

; otaceni smeru pohledu postavy - pro sledovani aktualni orientace postavy
(defun otoc_pohled_doleva(pohled)
  (cond ((eq pohled 'nahoru)  'doleva)
        ((eq pohled 'dolu)    'doprava)
        ((eq pohled 'doleva)  'dolu)
        ((eq pohled 'doprava) 'nahoru)
        (nil)
  )
)
(defun otoc_pohled_dozadu(pohled)  (otoc_pohled_doleva (otoc_pohled_doleva pohled)) )
(defun otoc_pohled_doprava(pohled) (otoc_pohled_doleva (otoc_pohled_dozadu pohled)) )

; prevody a upravy smerovych symbolu
(defun opacny_posun(pohled)
  (cond ((eq pohled 'doprava) 'doleva)
        ((eq pohled 'doleva)  'doprava)
        ((eq pohled 'dopredu) 'dozadu)
        ((eq pohled 'dozadu)  'dopredu)
        (nil)
  )
)
(defun preved_otoceni_na_posun(pohyb)
  (cond ((eq pohyb 'otoc_doleva)  'doleva)
        ((eq pohyb 'otoc_doprava) 'doprava)
        ((eq pohyb 'otoc_dozadu)  'dozadu)
        (nil)
  )
)
(defun preved_posun_na_otoceni(pohyb)
  (cond ((eq pohyb 'doleva)  'otoc_doleva)
        ((eq pohyb 'doprava) 'otoc_doprava)
        ((eq pohyb 'dozadu)  'otoc_dozadu)
        (nil)
  )
)

; prace se seznamem moznosti
(defun uprav(moznost co kolik)
  (cond ((null moznost) nil)
        ((eq co (caar moznost)) (rplacd (car moznost) (list (+ (cadar moznost) kolik))))
        ((uprav (cdr moznost) co kolik))
  )
)
(defun cti_moznost(moznosti co)
  (cond ((null moznosti) nil)
        ((eq co (caar moznosti)) (cadar moznosti))
        ((cti_moznost (cdr moznosti) co))
  )
)
(defun reset_moznosti(moznosti)
  (cond ((null moznosti) nil)
        ((rplacd (car moznosti) (list 0))
         (reset_moznosti (cdr moznosti))
        )
  )
)
(defun vyber(moznost)
  (cond ((null moznost) nil)
        ((null (cdr moznost)) (car moznost))
        ((let ((minimum (vyber (cdr moznost))))
              (cond ((>= (cadar moznost) (cadr minimum)) (car moznost))
                    (minimum)
              )
        ))
  )
)

; ladici vypisy
(defun vypis_moznosti(moznosti)
  (cond ((null moznosti) (terpri))
        ((princ (caar moznosti))
         (princ "=")
         (princ (cadar moznosti))
         (princ " ")
         (vypis_moznosti (cdr moznosti))
        )
  )
)
(defmethod vypis_radek((self trida_tahovy_objekt_postava_valeko)
                       (radek integer) (sloupec integer) (zbyva integer))
  (cond ((zerop zbyva) t)
        (t (princ (cadr (assoc (mapa_rel self
                                         (make-instance 'trida_pozice :x sloupec :y radek))
                 '((chodba "_") (zed "X") (klic "K") (strela "S") (vychod "!") (nil " "))
           )))
           (vypis_radek self radek (+ sloupec 1) (- zbyva 1))
        )
  )
)
(defmethod vypis_radky((self trida_tahovy_objekt_postava_valeko)
                       (sirka integer) (radek integer) (zbyva integer))
  (cond ((zerop zbyva) t)
        ((vypis_radek self radek (- 0 (/ (- sirka 1) 2)) sirka)
         (terpri)
         (vypis_radky self sirka (- radek 1) (- zbyva 1))
        )
  )
)
(defmethod vypis_mapu((self trida_tahovy_objekt_postava_valeko))
  (let ((vyska (vyska (slot_viditelna_mapa (slot_parametry self))))
        (sirka (sirka (slot_viditelna_mapa (slot_parametry self))))
       )
       (terpri)
       (vypis_radky self sirka (- vyska 1) vyska)
  )
)
(defun vypis_prisery(seznam)
  (cond ((null seznam) nil)
        ((princ "[") (princ (slot_x (slot_pozice (car seznam))))
         (princ ",") (princ (slot_y (slot_pozice (car seznam))))
         (princ "] ")
         (vypis_prisery (cdr seznam))
        )
  )
)

; vrati hodnotu z viditelne mapy - relativne
(defmethod mapa_rel((self trida_tahovy_objekt_postava_valeko) (pozice trida_pozice))
  (let ((mapa  (slot_viditelna_mapa (slot_parametry self)))
        (vyska (vyska (slot_viditelna_mapa (slot_parametry self))))
        (sirka (sirka (slot_viditelna_mapa (slot_parametry self))))
       )
       (cond ((>= (slot_y pozice) vyska) nil) ; nil mimo mapu misto zdi
             ((get_pole mapa
                        (soucet pozice
                                (make-instance 'trida_pozice :x (/ (- sirka 1) 2) :y 0)
             ))         )
       )
  )
)

; vrati hodnotu z viditelne mapy - na zaklade smeru a vzdalenosti
(defmethod mapa((self trida_tahovy_objekt_postava_valeko)
                (smer symbol) (velikost integer))
  (mapa_rel self (vektor smer velikost))
)

; zjisti, zda je v seznamu dana pozice - pro hledani viditelnych priser a strel
(defun hledej_pozici(seznam pozice)
 (cond ((null seznam) nil)
       ((rovnost pozice (slot_pozice (car seznam))) t)
       ((hledej_pozici (cdr seznam) pozice))
 )
)

; je prisera v tomto smeru v teto vzdalenosti ?
(defmethod prisera((self trida_tahovy_objekt_postava_valeko)
                   (smer symbol) (vzdalenost integer))
  (hledej_pozici (slot_viditelne_prisery (slot_parametry self))
                 (vektor smer vzdalenost)
) )

; je strela v tomto smeru v teto vzdalenosti ?
(defmethod strela((self trida_tahovy_objekt_postava_valeko)
                   (smer symbol) (vzdalenost integer))
  (hledej_pozici (slot_viditelne_strely (slot_parametry self))
                 (vektor smer vzdalenost)
) )

; nastavi hodnotu v zapamatovane mape
(defmethod cti_navstiveno((self trida_tahovy_objekt_postava_valeko)
                      (pozice trida_pozice))
 (aref (slot_navstiveno self) (+ (slot_y pozice) 100) (+ (slot_x pozice) 100) )
)

; cte hodnotu v zapamatovane mape
(defmethod nastav_navstiveno((self trida_tahovy_objekt_postava_valeko)
                             (pozice trida_pozice)
                             (hodnota integer))
 (setf (aref (slot_navstiveno self) (+ (slot_y pozice) 100) (+ (slot_x pozice) 100) )
       hodnota
 )
 (setf (slot_x_min self) (min (slot_x pozice) (slot_x_min self)))
 (setf (slot_x_max self) (max (slot_x pozice) (slot_x_max self)))
 (setf (slot_y_min self) (min (slot_y pozice) (slot_y_min self)))
 (setf (slot_y_max self) (max (slot_y pozice) (slot_y_max self)))
)

; zvysi hodnotu v zapamatovane mape a nastavi posledni pozici
(defmethod navstiv((self trida_tahovy_objekt_postava_valeko)
                   (pozice trida_pozice))
 (cond ((rovnost pozice (slot_posl_pozice self)) nil)
       ((nastav_navstiveno self pozice (+ (cti_navstiveno self pozice) 1))
        (setf (slot_posl_pozice self) pozice)
       )
 )
)

; uz jsme prohledli dost ? (nebo dochazeji tahy ci zivoty)
(defmethod hotovo((self trida_tahovy_objekt_postava_valeko))
 (or (< (slot_posl_zbyva self) 5)
     (> (slot_tahu self) 9000)
     (< (slot_pocet_zivotu (ja self)) 2)
)
)

; pomocna funkce
(defmethod bylo_navstiveno((self trida_tahovy_objekt_postava_valeko)
                             (pozice trida_pozice))
  (let ((hodnota (cti_navstiveno self pozice)))
       (and (> hodnota 0) (< hodnota 100))
  )
)

; kriterium pro skonceni prochazeni bludistem :
;  pocet poli, kde jsme nebyli, sousedicich s alespon jednim, kde jsme byli
(defmethod zbyva((self trida_tahovy_objekt_postava_valeko))
 (let
  ((count 0))
  (dotimes (x (+ 1 (- (slot_x_max self) (slot_x_min self))))
   (dotimes (y (+ 1 (- (slot_y_max self) (slot_y_min self))))
    (let
     ((pozice (make-instance 'trida_pozice
                             :x (+ x (slot_x_min self))
                             :y (+ y (slot_y_min self))
     ))       )
     (setf count (+ count 
      (cond
       ((not (zerop (cti_navstiveno self pozice))) 0)
       ((bylo_navstiveno self (soucet pozice (vektor 'dopredu 1))) 1)
       ((bylo_navstiveno self (soucet pozice (vektor 'dozadu  1))) 1)
       ((bylo_navstiveno self (soucet pozice (vektor 'doleva  1))) 1)
       ((bylo_navstiveno self (soucet pozice (vektor 'doprava 1))) 1)
      (0)
     )))
    )
   )
  )
  count
 )
)

; rekurzivne se rozhledne po viditelne mape (smerem dopredu a pote do stran)
; t = nevime nic, nil = je to slepa ulicka, dopredu = je tu vychod
(defmethod rozhled_dopredu((self trida_tahovy_objekt_postava_valeko)
                           (pozice trida_pozice) (pole array))
 (cond ((null (mapa_rel self pozice)) t)             ; konci v neznamu
       ((eq 'ZED (mapa_rel self pozice)) nil)        ; konci zdi - muze to byt slepa ulicka
       ((and (hotovo self) (eq 'VYCHOD (mapa_rel self pozice)) 'dopredu))
                                                     ; slepa ulicka s vychodem neni slepa
       ((zerop (slot_y pozice)) nil)                 ; nezajima nas primo vlevo a vpravo
       ((aref pole (+ 25 (slot_x pozice)) (+ 25 (slot_y pozice))) nil)
       ((setf (aref pole (+ 25 (slot_x pozice)) (+ 25 (slot_y pozice))) t)
        (cond ((rozhled_dopredu self (soucet pozice (vektor 'dopredu 1)) pole))
              ((rozhled_dopredu self (soucet pozice (vektor 'doleva  1)) pole))
              ((rozhled_dopredu self (soucet pozice (vektor 'doprava 1)) pole))
              ((rozhled_dopredu self (soucet pozice (vektor 'dozadu  1)) pole))
              (nil)
          )
       )
 )
)

; rekurzivne se rozhledne po viditelne mape (jednim smerem)
; nil = nic zajimaveho, nas smer = je tam vychod
(defmethod rozhled_do_stran((self trida_tahovy_objekt_postava_valeko)
                            (pozice trida_pozice) (smer symbol))
 (cond ((member (mapa_rel self pozice) '(nil ZED)) nil) ; konci
       ((and (hotovo self) (eq 'VYCHOD (mapa_rel self pozice)) smer))
                                                        ; vychod je timto smerem
       ((rozhled_do_stran self (soucet pozice (vektor smer 1)) smer))
 )
)

; rozhledne se po viditelne mape a vrati, co je zajimaveho
; vraci: t = nevime nic, nil = je to slepa ulicka, nejaky smer = je tu vychod
(defmethod rozhled((self trida_tahovy_objekt_postava_valeko))
 (cond ((rozhled_do_stran self vektor_doleva  'doleva))
       ((rozhled_do_stran self vektor_doprava 'doprava))
       ((rozhled_dopredu self vektor_nahoru (make-array '(50 50) :initial-element nil)))
 )
)

; vrati absolutni pozici, na kterou by se postava dostala timto smerem
(defmethod pohyb((self trida_tahovy_objekt_postava_valeko)
                 (pohyb symbol) (vzdalenost integer))
 (soucet (slot_pozice self)
         (otoc_vektor (slot_pohled self) (vektor pohyb vzdalenost))
 )
)

; "provede" tah tesne pred navracenim hodnoty - aktualizuje vnitrni promenne objektu
(defmethod aktualizuj((self trida_tahovy_objekt_postava_valeko) (vybrano symbol))
 (setf (slot_posl_akce self) vybrano)
 (cond ((member vybrano '(otoc_doleva otoc_doprava otoc_dozadu))
        (setf (slot_posl_otoceni self) vybrano))
 )
 (cond
  ; pohyb
  ((vektor vybrano 1)
   (setf (slot_pozice self) (pohyb self vybrano 1))
  )
  ; otaceni
  ((eq vybrano 'otoc_doleva)
   (setf (slot_pohled self) (otoc_pohled_doleva  (slot_pohled self)))
  )
  ((eq vybrano 'otoc_doprava)
   (setf (slot_pohled self) (otoc_pohled_doprava (slot_pohled self))) 
  )
  ((eq vybrano 'otoc_dozadu)
   (setf (slot_pohled self) (otoc_pohled_dozadu  (slot_pohled self)))
  )
 )
)

; vrati objekt vlastni postavy
(defmethod ja((self trida_tahovy_objekt_postava_valeko))
  (slot_vlastni_postava (slot_parametry self))
)

; hlavni tahova funkce
(defmethod tahova_funkce((self trida_tahovy_objekt_postava_valeko)
                         (parametry trida_parametry_tahu))
 
 (setf (slot_parametry self) parametry)         ; ulozit parametry do objektu
 (navstiv self (slot_pozice self))              ; zvysit pocitadlo navsetv u aktualni pozice
 (setf (slot_tahu self) (+ 1 (slot_tahu self))) ; zvysit pocitadlo tahu
 (let ((moznost '((dopredu 0) (doleva 0) (doprava 0) (dozadu 0)
                   (otoc_doleva 0) (otoc_doprava 0) (otoc_dozadu 0)
                   (strela 0) (stop 0))))
  (reset_moznosti moznost)

  (uprav moznost 'dopredu 2)
  (uprav moznost 'dozadu -20)
  (uprav moznost 'doleva -2)
  (uprav moznost 'doprava -2)
  (uprav moznost 'otoc_doleva 1)
  (uprav moznost 'otoc_doprava 1)
  (uprav moznost 'otoc_dozadu -2)
  (uprav moznost 'stop -20)

  ; zapamatovat si, kde jsou zdi kolem mne
  (mapc #'(lambda(smer) (cond ((eq 'ZED (mapa self smer 1))
                               (nastav_navstiveno self (pohyb self smer 1) 100))
          )             )
         '(dopredu doleva doprava)
  )
  ; prepocitat neprozkoumana pole - jednou za 10 tahu
  (cond ((zerop (mod (slot_tahu self) 10))
         (setf (slot_posl_zbyva self) (zbyva self)))
  )
  ; jit do vychodu, pokud je hned vedle a uz je proc
  (mapc #'(lambda(smer) (cond ((eq 'VYCHOD (mapa self smer 1))
                               (cond ((hotovo self) (uprav moznost smer 100))
                                     ((uprav moznost smer -100))
                               )
                              )
                        )
          )
         '(dopredu doleva doprava)
  )
  ; nechodit a neotacet se do smeru, kde je strela
  (mapc #'(lambda(smer) (cond ((strela self smer 1)
                               (uprav moznost smer -100)
                               (uprav moznost (preved_posun_na_otoceni smer) -100))
          )             )
         '(dopredu doleva doprava)
  )
  ; nechodit do smeru, kde je prisera, naopak jit rychle nekam jinam
  ; pokud nemame strely, ani se neotacet
  (mapc #'(lambda(smer) (cond ((prisera self smer 1)
                               (uprav moznost smer -100)
                               (uprav moznost (preved_posun_na_otoceni smer) -100)
                               (mapc #'(lambda(smer) (uprav moznost smer 20))
                                      '(dopredu doleva doprava dozadu))
                              )
          )             )
         '(dopredu doleva doprava)
  )
  ; kdyz nemame strely, bojime se priser
  ; nechodit a neotacet se do smeru, kde je prisera ve vzdalenosti 2 nebo 3
  (mapc #'(lambda(vzdalenost)
            (mapc #'(lambda(smer)
                     (cond ((prisera self smer vzdalenost)
                            (uprav moznost smer -40)
                            (cond ((zerop (slot_pocet_strel (ja self)))
                                   (uprav moznost (preved_posun_na_otoceni smer) -40))
                                  ((uprav moznost (preved_posun_na_otoceni smer) 10))
                            )
                           )
                     )
                   )
                   '(dopredu doleva doprava)
            )
          )
         '(2 3)
  )
  ; prisera sikmo vlevo
  (cond ((prisera self 'dopredu_doleva 1)
         (uprav moznost 'dopredu -50)
         (uprav moznost 'doleva -50)
         (uprav moznost 'otoc_doleva -50)
        )
  )
  ; prisera sikmo vpravo
  (cond ((prisera self 'dopredu_doprava 1)
         (uprav moznost 'dopredu -50)
         (uprav moznost 'doprava -50)
         (uprav moznost 'otoc_doprava -50)
        )
  )
  ; nezustavat na miste
  (cond ((member (slot_posl_akce self)
                '(otoc_doleva otoc_doprava otoc_dozadu stop))
         (mapc #'(lambda(smer) (uprav moznost smer -20))
                '(otoc_doleva otoc_doprava otoc_dozadu stop))
        )
  )
  (uprav moznost (slot_posl_otoceni self) -1)
  ; nevracet se, odkud jsem prisli
  (cond ((mapc #'(lambda(smer)
                   (cond ((eq smer (slot_posl_akce self))
                          (uprav moznost (opacny_posun smer) -20)
                          (uprav moznost (preved_posun_na_otoceni (opacny_posun smer)) -20))
                   )
                 )
                '(doleva doprava dopredu dozadu))
        )
  )
  ; nestrilet, kdyz neni cim
  (cond ((zerop (slot_pocet_strel (ja self)))
         (uprav moznost 'strela -200))
  )
  ; nestrilet, kdyz je strela uz pred nami
  (cond ((or (strela self 'dopredu 2) (strela self 'dopredu 3)) (uprav moznost 'strela -200)))
  ; nestrilet dvakrat za sebou a nechodit za strelou
  (cond ((eq 'strela (slot_posl_akce self))
         (uprav moznost 'strela -100)
         (uprav moznost 'dopredu -100)
         (uprav moznost 'stop 25))
  )
  ; zastrelit priseru
  (cond ((prisera self 'dopredu 1) (uprav moznost 'strela 50))
        ((prisera self 'dopredu 2) (uprav moznost 'strela 20))
        ((prisera self 'dopredu 3) (uprav moznost 'strela 20))
        ((uprav moznost 'strela -100))
  )
  ; rozhlednout se
  (let ((rozhled (rozhled self)))
       (cond ((null rozhled)                     ; slepa ulicka
              (uprav moznost 'dopredu -15)
              (uprav moznost 'otoc_doleva 5)
              (uprav moznost 'otoc_doprava 5)
              (uprav moznost 'otoc_dozadu 5)
              (cond ((not (eq 'ZED (mapa self 'dopredu 1)))
                     (nastav_navstiveno self (pohyb self 'dopredu 1) 5))
              ))
             ((eq rozhled 'dopredu)              ; je videt vychod vepredu
              (uprav moznost 'dopredu 10)
             )
             ((member rozhled '(doleva doprava)) ; je videt vychod do strany
              (uprav moznost (preved_posun_na_otoceni rozhled) 5)
             )
       )
  )
  ; nechodit a neotacet se tam, kde uz jsme byli
  (mapc #'(lambda(smer) 
            (let ((hodnota (* -2 (cti_navstiveno self (pohyb self smer 1)))))
                 (uprav moznost smer hodnota)
                 (uprav moznost (preved_posun_na_otoceni smer) hodnota)
            )
          )
         '(dopredu doleva doprava dozadu)
  )
  ; ale prekonat jiz proslou ulicku, pokud jsme za ni jeste nebyli
  (mapc #'(lambda(smer)
            (cond ((zerop (cti_navstiveno self (pohyb self smer 2)))
                   (uprav moznost smer 4)
                   (uprav moznost (preved_posun_na_otoceni smer) 4))
            )
          )
         '(dopredu doleva doprava dozadu)
  )
  ; nastavit pole jako navstivene, pokud je to nenavstivena chodba, a za ni uz jsme byli
  (mapc #'(lambda(smer) (cond ((and (eq 'CHODBA (mapa self smer 1))
;                                    (eq 'CHODBA (mapa self smer 2))
                                    (zerop (cti_navstiveno self (pohyb self smer 1)))
                                    (> (cti_navstiveno self (pohyb self smer 2)) 0)
                               )
                               (nastav_navstiveno self (pohyb self smer 1) 1))
          )             )
         '(dopredu doleva doprava)
  )

;  (vypis_prisery (slot_viditelne_prisery parametry))
;  (vypis_mapu self)
;  (princ " navstev:") (princ (cti_navstiveno self (slot_pozice self)))
;  (princ " zivotu:") (princ (slot_pocet_zivotu (ja self)))
;  (princ " strel:")  (princ (slot_pocet_strel (ja self)))
;  (terpri)
;  (vypis_moznosti moznost)
;  (terpri)

;  (cond ((zerop (mod (slot_tahu self) 10))
;         (princ " tahu:")
;         (princ (slot_tahu self))
;         (princ " zbyva:")
;         (princ (slot_posl_zbyva self))
;         (terpri)
;        )
;  )

  ; vybereme moznost s nejvyssim ohodnocenim - a tu provedeme
  (let ((vybrano (car (vyber moznost))))
;       (princ "Vybrano:") (princ vybrano) (terpri)
       (aktualizuj self vybrano)
       (make-instance 'trida_tah :akce vybrano)
  )
 )
)
